/**
 * Java
 *
 * Copyright 2012 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY. Use is subject to license terms.
 */
package com.is2t.mwt.tutorial.ex11;

import ej.microui.io.GraphicsContext;
import ej.microui.io.Pointer;
import ej.mwt.Renderable;
import ej.mwt.Widget;
import ej.mwt.rendering.Look;
import ej.mwt.rendering.WidgetRenderer;

public class ScaleRenderer extends WidgetRenderer implements ScaleRendererContract {

	private static final int STATE_IDLE = 0;
	private static final int STATE_ADJUSTING = 1;

	private static final int CENTER_STRIP_THICKNESS = 4;
	private static final int THUMB_SIZE = 13;
	
	public Class getManagedType() {
		return Scale.class;
	}

	public int getPreferredContentWidth(Widget widget) {
		return getThumbSize();
	}

	public int getPreferredContentHeight(Widget widget) {
		Scale scale = (Scale)widget;
		return scale.getBoundMax() - scale.getBoundMin() + getThumbSize();
	}

	public void render(GraphicsContext g, Renderable renderable) {
		Scale scale = (Scale)renderable;
		int width = scale.getWidth();
		int height = scale.getHeight();
		boolean isDragging = scale.getRenderingState() == STATE_ADJUSTING;

		int background = getLook().getProperty(scale.isEnabled()?
				Look.GET_BACKGROUND_COLOR_CONTENT:Look.GET_BACKGROUND_COLOR_DISABLED);
		int foreground = getLook().getProperty(Look.GET_FOREGROUND_COLOR_DEFAULT);
		int thumbColor = isDragging?
				getLook().getProperty(Look.GET_BACKGROUND_COLOR_SELECTION):background;
		int borderColor = getLook().getProperty(Look.GET_BORDER_COLOR_DEFAULT);

		g.setColor(background);
		g.fillRect(0, 0, width, height);

		g.setColor(foreground);
		int centerLineOffset = width/2;
		int centerStripOffset = getCenterStripThickness()/2;
		int thumbOffset = getThumbSize()/2;
		g.fillRect(centerLineOffset-centerStripOffset, 0, getCenterStripThickness(), height);
	
		g.setColor(thumbColor);
		int position = computePositionFromValue(height-getThumbSize(), scale.getBoundMin(),
				scale.getBoundMax(), scale.getValue());
		g.fillCircle(centerLineOffset-thumbOffset, position, getThumbSize());
		g.setColor(borderColor);
		g.drawCircle(centerLineOffset-thumbOffset, position, getThumbSize());
	
		if(scale.hasFocus()) {
			g.setStrokeStyle(GraphicsContext.DOTTED);
			g.drawRect(0, 0, width-1, height-1);
		}
	}

	public int getMargin() {
		return 4;
	}
	
	public boolean processPointerEvent(Scale scale, int action, int x,int y) {
		switch (action) {
		case Pointer.PRESSED:
			if (thumbContains(scale, x, y)) {
				scale.setRenderingState(STATE_ADJUSTING);
				scale.repaint();
			}
			return true;
		case Pointer.RELEASED:
			scale.setRenderingState(STATE_IDLE);
			scale.repaint();
			return true;
		case Pointer.DRAGGED:
			if (scale.getRenderingState() == STATE_ADJUSTING) {
				scale.setValue(computeValue(scale, x, y));
			}
			return true;
		}
		return false;
	}

	private boolean thumbContains(Scale scale, int x, int y) {
		int height = scale.getHeight();
		int min = scale.getBoundMin();
		int max = scale.getBoundMax();
		int selection = scale.getValue();
		int thumbTopPosition = computePositionFromValue(height-getThumbSize(),
				min, max, selection);
		return y >= thumbTopPosition && y<= thumbTopPosition+getThumbSize();
	}

	private int computeValue(Scale scale, int x, int y) {
		int min = scale.getBoundMin();
		int max = scale.getBoundMax();
		return computeValueFromPosition(scale.getHeight(), min, max, y);
	}
	
	private int computePositionFromValue(int scaleSize, int min, int max, int value) {
		return scaleSize - (scaleSize*Math.abs(value-min) / Math.abs(max-min));
	}

	private int computeValueFromPosition(int scaleSize, int min,
			int max, int selectedPosition) {
		return ((scaleSize-selectedPosition-getThumbSize()/2)*
				Math.abs(max-min)/(scaleSize-getThumbSize()))+min;
	}

	private int getThumbSize() {
		return THUMB_SIZE;
	}
	
	private int getCenterStripThickness() {
		return CENTER_STRIP_THICKNESS;
	}
}
