/**
 * Java
 *
 * Copyright 2012 IS2T. All rights reserved.
 * For demonstration purpose only.
 * IS2T PROPRIETARY. Use is subject to license terms.
 */
package com.is2t.mwt.tutorial.ex13;

import ej.microui.io.GraphicsContext;
import ej.mwt.Renderable;
import ej.mwt.rendering.Look;

public class HangingBoxRenderer extends ScaleRenderer {

	public static final int STYLE = 2;

	private static final int CENTER_STRIP_THICKNESS = 2;
	private static final int THUMB_SIZE = 19;
	
	public int getManagedStyle() {
		return STYLE;
	}

	public void render(GraphicsContext g, Renderable renderable) {
		Scale scale = (Scale)renderable;
		int width = scale.getWidth();
		int height = scale.getHeight();
		boolean isDragging = scale.getRenderingState() == STATE_ADJUSTING;

		int background = getLook().getProperty(scale.isEnabled()?
				Look.GET_BACKGROUND_COLOR_CONTENT:Look.GET_BACKGROUND_COLOR_DISABLED);
		int foreground = getLook().getProperty(Look.GET_FOREGROUND_COLOR_DEFAULT);
		int thumbColor = isDragging?
				getLook().getProperty(Look.GET_BACKGROUND_COLOR_SELECTION):
				scale.isEnabled()?
					getLook().getProperty(Look.GET_BACKGROUND_COLOR_FOCUSED):foreground;
		int borderColor = getLook().getProperty(Look.GET_BORDER_COLOR_DEFAULT);

		if (!scale.isTransparent()) {
			g.setColor(background);
			g.fillRect(0, 0, width, height);
		}

		g.setColor(foreground);
		int centerLineOffset = width/2;
		int centerStripOffset = getCenterStripThickness()/2;
		int thumbOffset = getThumbSize()/2;
		int position = computePositionFromValue(height-getThumbSize(), scale.getBoundMin(),
				scale.getBoundMax(), scale.getValue());
		g.fillRect(centerLineOffset-centerStripOffset, 0, getCenterStripThickness(), position);
	
		g.setColor(thumbColor);
		g.fillRect(centerLineOffset-thumbOffset, position, getThumbSize(), getThumbSize());
	
		if(scale.hasFocus()) {
			g.setColor(borderColor);
			g.setStrokeStyle(GraphicsContext.DOTTED);
			g.drawRect(0, 0, width-1, height-1);
		}
	}
	
	protected int getThumbSize() {
		return THUMB_SIZE;
	}
	
	protected int getCenterStripThickness() {
		return CENTER_STRIP_THICKNESS;
	}
}