/**
 * Java
 *
 * Copyright 2012 IS2T. All rights reserved.
 * For demonstration purpose only
 * IS2T PROPRIETARY. Use is subject to license terms.
 */
package com.is2t.mwt.tutorial.ex15;

import ej.microui.io.DisplayFont;
import ej.microui.io.GraphicsContext;
import ej.mwt.Renderable;
import ej.mwt.Widget;
import ej.mwt.rendering.Look;
import ej.mwt.rendering.WidgetRenderer;

public class NumericReadoutRenderer extends WidgetRenderer implements ScaleRendererContract {
	
	public static final int STYLE = 3;

	public Class getManagedType() {
		return Scale.class;
	}

	public int getManagedStyle() {
		return STYLE;
	}

	public int getPreferredContentWidth(Widget widget) {
		Scale scale = (Scale)widget;
		String boundMaxAsString = Integer.toString(scale.getBoundMax());
		String boundMinAsString = Integer.toString(scale.getBoundMin());
		String longestString = boundMaxAsString.length()>boundMinAsString.length() ?
				boundMaxAsString : boundMinAsString;
		return getNormalFont().stringWidth(longestString);
	}

	public int getPreferredContentHeight(Widget widget) {
		return getNormalFont().getHeight();
	}

	public boolean isTransparentPreferred() {
		return false;
	}

	public void render(GraphicsContext g, Renderable renderable) {
		Scale scale = (Scale)renderable;
		int width = scale.getWidth();
		int height = scale.getHeight();
		int background = getLook().getProperty(scale.isEnabled()?
				Look.GET_BACKGROUND_COLOR_CONTENT:Look.GET_BACKGROUND_COLOR_DISABLED);
		int foreground = getLook().getProperty(Look.GET_FOREGROUND_COLOR_DEFAULT);
		int borderColor = getLook().getProperty(Look.GET_BORDER_COLOR_DEFAULT);

		if (!scale.isTransparent()) {
			g.setColor(background);
			g.fillRect(0, 0, width, height);
		}

		g.setColor(foreground);
		DisplayFont font = getLook().getFonts()[getLook().getProperty(Look.GET_FONT_INDEX_DEFAULT)];
		g.setFont(font);
		String valueAsString = Integer.toString(scale.getValue());
		g.drawString(valueAsString , width-getPadding(), height/2, GraphicsContext.VCENTER | GraphicsContext.RIGHT);
		g.setColor(borderColor);
		g.drawRect(0, 0, width-1, height-1);
	
		if(scale.hasFocus()) {
			g.setStrokeStyle(GraphicsContext.DOTTED);
			g.drawRect(2, 2, width-5, height-5);
		}
	}

	public int getMargin() {
		return 2;
	}

	public int getPadding() {
		return 4;
	}

	public boolean processPointerEvent(Scale scale, int action, int x,int y) {
		// don't handle any pointer events
		return false;
	}
	
	private DisplayFont getNormalFont() {
		return getLook().getFonts()[getLook().getProperty(
				Look.GET_FONT_INDEX_DEFAULT)];
	}
}
