/**
 * Java
 *
 * Copyright 2012 IS2T. All rights reserved.
 * For demonstration purpose only
 * IS2T PROPRIETARY. Use is subject to license terms.
 */
package com.is2t.mwt.tutorial.ex16.widget;

import ej.microui.io.GraphicsContext;
import ej.mwt.Renderable;
import ej.mwt.rendering.Look;

public class HangingBoxRenderer extends ScaleRenderer {

	public static final int STYLE = 2;

	private static final int CENTER_STRIP_THICKNESS = 2;
	private static final int THUMB_SIZE = 19;
	
	public int getManagedStyle() {
		return STYLE;
	}

	public void render(GraphicsContext g, Renderable renderable) {
		Scale scale = (Scale)renderable;
		int width = scale.getWidth();
		int height = scale.getHeight();
		boolean isDragging = scale.getRenderingState() == STATE_ADJUSTING;

		int foreground = getLook().getProperty(Look.GET_FOREGROUND_COLOR_DEFAULT);
		int thumbColor = isDragging?
				getLook().getProperty(Look.GET_BACKGROUND_COLOR_SELECTION):
				scale.isEnabled()?
					getLook().getProperty(Look.GET_BACKGROUND_COLOR_FOCUSED):foreground;

		fillBackground(g, scale);

		g.setColor(foreground);
		int centerLineOffset = width/2;
		int centerStripOffset = getCenterStripThickness()/2;
		int thumbOffset = getThumbSize()/2;
		int position = computePositionFromValue(height-getThumbSize(), scale.getBoundMin(),
				scale.getBoundMax(), scale.getValue());
		g.fillRect(centerLineOffset-centerStripOffset, 0, getCenterStripThickness(), position);
	
		g.setColor(thumbColor);
		g.fillRect(centerLineOffset-thumbOffset, position, getThumbSize(), getThumbSize());
	
		drawFocusOutline(g, scale, 0);
	}
	
	protected int getThumbSize() {
		return THUMB_SIZE;
	}
	
	protected int getCenterStripThickness() {
		return CENTER_STRIP_THICKNESS;
	}
}