/**
 * Java
 *
 * Copyright 2012 IS2T. All rights reserved.
 * For demonstration purpose only
 * IS2T PROPRIETARY. Use is subject to license terms.
 */
package com.is2t.mwt.tutorial.ex16.widget;

import ej.microui.Event;
import ej.microui.Listener;
import ej.microui.io.Pointer;
import ej.mwt.Widget;

/**
 * A scale is a widget that holds an integer value between two bounds.
 */
public class Scale extends Widget implements Listener {
	
	private int value;
	private int lowerBound;
	private int upperBound;
	private int renderingState;
	private Listener listener;
	private int style;
	
	public Scale(int lowerBound, int upperBound, int style) {
		super();
		this.lowerBound = lowerBound;
		this.upperBound = upperBound;
		this.style = style;
	}

	public int getStyle() {
		return style;
	}

	public int getBoundMin() {
		return lowerBound;
	}

	public int getBoundMax() {
		return upperBound;
	}

	public int getValue() {
		return value;
	}

	public void setValue(int val) {
		if (updateValue(val) && listener != null)
			listener.performAction(value);
	}

	public boolean isTransparent() {
		ScaleRendererContract renderer = (ScaleRendererContract) getRenderer();
		return renderer == null || renderer.isTransparentPreferred();
	}

	public void setListener(Listener listener) {
		this.listener = listener;
	}

	public boolean handleEvent(int event) {
		int type = Event.getType(event);
		if (type == Event.POINTER) {
			ScaleRendererContract renderer = (ScaleRendererContract) getRenderer();
			int action = Pointer.getAction(event);
			Pointer pointer = (Pointer)Event.getGenerator(event);
			int x = pointer.getX();
			int y = pointer.getY();
			return renderer.processPointerEvent(this, action, x - getAbsoluteX(), y - getAbsoluteY());
		}
		return super.handleEvent(event);
	}

	// for use by renderer
	
	public int getRenderingState() {
		return renderingState;
	}

	public void setRenderingState(int renderingState) {
		this.renderingState = renderingState;
	}

	// listener implementation
	
	public void performAction() {
		// no action
	}

	public void performAction(int value) {
		updateValue(value);
	}

	public void performAction(int value, Object object) {
		updateValue(value);
	}

	protected boolean updateValue(int val) {
		int newValue = Math.min(Math.max(val, lowerBound), upperBound);
		if (newValue == value) return false;
		value = newValue;
		repaint();
		return true;
	}
}
